@@ -1,5 +1,5 @@
 use crate::constants::{
-    MAX_I128_REPR, MAX_PRECISION, POWERS_10, SCALE_MASK, SCALE_SHIFT, SIGN_MASK, SIGN_SHIFT, U32_MASK, U8_MASK,
+    MAX_I128_REPR, MAX_PRECISION_U32, POWERS_10, SCALE_MASK, SCALE_SHIFT, SIGN_MASK, SIGN_SHIFT, U32_MASK, U8_MASK,
     UNSIGN_MASK,
 };
 use crate::ops;
@@ -263,7 +263,7 @@ impl Decimal {
     /// assert!(max.is_err());
     /// ```
     pub const fn try_new(num: i64, scale: u32) -> crate::Result<Decimal> {
-        if scale > MAX_PRECISION {
+        if scale > MAX_PRECISION_U32 {
             return Err(Error::ScaleExceedsMaximumPrecision(scale));
         }
         let flags: u32 = scale << SCALE_SHIFT;
@@ -323,7 +323,7 @@ impl Decimal {
     /// assert!(max.is_err());
     /// ```
     pub const fn try_from_i128_with_scale(num: i128, scale: u32) -> crate::Result<Decimal> {
-        if scale > MAX_PRECISION {
+        if scale > MAX_PRECISION_U32 {
             return Err(Error::ScaleExceedsMaximumPrecision(scale));
         }
         let mut neg = false;
@@ -382,7 +382,7 @@ impl Decimal {
                 } else {
                     negative
                 },
-                scale % (MAX_PRECISION + 1),
+                scale % (MAX_PRECISION_U32 + 1),
             ),
         }
     }
@@ -442,7 +442,7 @@ impl Decimal {
                 // we've parsed 1.2 as the base and 10 as the exponent. To represent this within a
                 // Decimal type we effectively store the mantissa as 12,000,000,000 and scale as
                 // zero.
-                if exp > MAX_PRECISION {
+                if exp > MAX_PRECISION_U32 {
                     return Err(Error::ScaleExceedsMaximumPrecision(exp));
                 }
                 let mut exp = exp as usize;
@@ -606,7 +606,7 @@ impl Decimal {
     /// assert_eq!(one.to_string(), "0.00001");
     /// ```
     pub fn set_scale(&mut self, scale: u32) -> Result<(), Error> {
-        if scale > MAX_PRECISION {
+        if scale > MAX_PRECISION_U32 {
             return Err(Error::ScaleExceedsMaximumPrecision(scale));
         }
         self.flags = (scale << SCALE_SHIFT) | (self.flags & SIGN_MASK);
@@ -691,13 +691,37 @@ impl Decimal {
     /// * Bytes 9-12: mid portion of `m`
     /// * Bytes 13-16: high portion of `m`
     #[must_use]
-    pub const fn deserialize(bytes: [u8; 16]) -> Decimal {
-        Decimal {
-            flags: (bytes[0] as u32) | (bytes[1] as u32) << 8 | (bytes[2] as u32) << 16 | (bytes[3] as u32) << 24,
+    pub fn deserialize(bytes: [u8; 16]) -> Decimal {
+        // We can bound flags by a bitwise mask to correspond to:
+        //   Bits 0-15: unused
+        //   Bits 16-23: Contains "e", a value between 0-28 that indicates the scale
+        //   Bits 24-30: unused
+        //   Bit 31: the sign of the Decimal value, 0 meaning positive and 1 meaning negative.
+        let mut raw = Decimal {
+            flags: ((bytes[0] as u32) | (bytes[1] as u32) << 8 | (bytes[2] as u32) << 16 | (bytes[3] as u32) << 24)
+                & 0x801F_0000,
             lo: (bytes[4] as u32) | (bytes[5] as u32) << 8 | (bytes[6] as u32) << 16 | (bytes[7] as u32) << 24,
             mid: (bytes[8] as u32) | (bytes[9] as u32) << 8 | (bytes[10] as u32) << 16 | (bytes[11] as u32) << 24,
             hi: (bytes[12] as u32) | (bytes[13] as u32) << 8 | (bytes[14] as u32) << 16 | (bytes[15] as u32) << 24,
+        };
+        // Scale must be bound to maximum precision. Only two values can be greater than this
+        if raw.scale() > MAX_PRECISION_U32 {
+            let mut bits = raw.mantissa_array3();
+            let remainder = match raw.scale() {
+                29 => crate::ops::array::div_by_1x(&mut bits, 1),
+                30 => crate::ops::array::div_by_1x(&mut bits, 2),
+                31 => crate::ops::array::div_by_1x(&mut bits, 3),
+                _ => 0,
+            };
+            if remainder >= 5 {
+                ops::array::add_one_internal(&mut bits);
+            }
+            raw.lo = bits[0];
+            raw.mid = bits[1];
+            raw.hi = bits[2];
+            raw.flags = flags(raw.is_sign_negative(), MAX_PRECISION_U32);
         }
+        raw
     }
 
     /// Returns `true` if the decimal is negative.
@@ -1189,6 +1213,9 @@ impl Decimal {
     /// assert_eq!(value.round_sf(6), Some(Decimal::from_str("305.459").unwrap()));
     /// assert_eq!(value.round_sf(7), Some(Decimal::from_str("305.4590").unwrap()));
     /// assert_eq!(Decimal::MAX.round_sf(1), None);
+    ///
+    /// let value = Decimal::from_str("0.012301").unwrap();
+    /// assert_eq!(value.round_sf(3), Some(Decimal::from_str("0.0123").unwrap()));
     /// ```
     #[must_use]
     pub fn round_sf(&self, digits: u32) -> Option<Decimal> {
@@ -1228,6 +1255,9 @@ impl Decimal {
     /// assert_eq!(value.round_sf_with_strategy(6, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("305.459").unwrap());
     /// assert_eq!(value.round_sf_with_strategy(7, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("305.4590").unwrap());
     /// assert_eq!(Decimal::MAX.round_sf_with_strategy(1, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("70000000000000000000000000000").unwrap());
+    ///
+    /// let value = Decimal::from_str("0.012301").unwrap();
+    /// assert_eq!(value.round_sf_with_strategy(3, RoundingStrategy::AwayFromZero), Some(Decimal::from_str("0.0124").unwrap()));
     /// ```
     #[must_use]
     pub fn round_sf_with_strategy(&self, digits: u32, strategy: RoundingStrategy) -> Option<Decimal> {
@@ -1244,21 +1274,18 @@ impl Decimal {
             let _remainder = ops::array::div_by_u32(&mut working, 10u32);
             mantissa_sf += 1;
             if working[2] == 0 && working[1] == 0 && working[0] == 1 {
+                mantissa_sf += 1;
                 break;
             }
         }
-
-        // Total significant figures in the full number also needs to include scale.
-        // e.g. mantissa 1, scale of 6 = 0.000001 (6 sf), mantissa 1234, scale of 3 = 1.234 (4 sf)
         let scale = self.scale();
-        let current_sf = if scale > mantissa_sf { scale } else { mantissa_sf };
 
-        match digits.cmp(&current_sf) {
+        match digits.cmp(&mantissa_sf) {
             Ordering::Greater => {
                 // If we're requesting a higher number of significant figures, we rescale
                 let mut array = [self.lo, self.mid, self.hi];
-                let mut value_scale = self.scale();
-                ops::array::rescale_internal(&mut array, &mut value_scale, scale + digits - current_sf);
+                let mut value_scale = scale;
+                ops::array::rescale_internal(&mut array, &mut value_scale, scale + digits - mantissa_sf);
                 Some(Decimal {
                     lo: array[0],
                     mid: array[1],
@@ -1268,15 +1295,17 @@ impl Decimal {
             }
             Ordering::Less => {
                 // We're requesting a lower number of significant digits.
-                let diff = current_sf - digits;
+                let diff = mantissa_sf - digits;
                 // If the diff is greater than the scale we're focused on the integral. Otherwise, we can
                 // just round.
                 if diff > scale {
                     use crate::constants::BIG_POWERS_10;
                     // We need to adjust the integral portion. This also should be rounded, consequently
                     // we reduce the number down, round it, and then scale back up.
-                    // E.g. If we have 305.459 scaling to a sf of 2. We first, reduce the number
+                    // E.g. If we have 305.459 scaling to a sf of 2 - we first reduce the number
                     // down to 30.5459, round it to 31 and then scale it back up to 310.
+                    // Likewise, if we have 12301 scaling to a sf of 3 - we first reduce the number
+                    // down to 123.01, round it to 123 and then scale it back up to 12300.
                     let mut num = *self;
                     let mut exp = (diff - scale) as usize;
                     while exp > 0 {
@@ -1291,7 +1320,7 @@ impl Decimal {
                         num = num.checked_div(pow)?;
                     }
                     let mut num = num.round_dp_with_strategy(0, strategy).trunc();
-                    let mut exp = (current_sf - digits - scale) as usize;
+                    let mut exp = (mantissa_sf - digits - scale) as usize;
                     while exp > 0 {
                         let pow;
                         if exp >= BIG_POWERS_10.len() {
@@ -1374,140 +1403,46 @@ impl Decimal {
         [self.lo, self.mid, self.hi, 0]
     }
 
-    fn base2_to_decimal(bits: &mut [u32; 3], exponent2: i32, positive: bool, is64: bool) -> Option<Self> {
-        // 2^exponent2 = (10^exponent2)/(5^exponent2)
-        //             = (5^-exponent2)*(10^exponent2)
-        let mut exponent5 = -exponent2;
-        let mut exponent10 = exponent2; // Ultimately, we want this for the scale
+    /// Parses a 32-bit float into a Decimal number whilst retaining any non-guaranteed precision.
+    ///
+    /// Typically when a float is parsed in Rust Decimal, any excess bits (after ~7.22 decimal points for
+    /// f32 as per IEEE-754) are removed due to any digits following this are considered an approximation
+    /// at best. This function bypasses this additional step and retains these excess bits.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use rust_decimal::prelude::*;
+    ///
+    /// // Usually floats are parsed leveraging float guarantees. i.e. 0.1_f32 => 0.1
+    /// assert_eq!("0.1", Decimal::from_f32(0.1_f32).unwrap().to_string());
+    ///
+    /// // Sometimes, we may want to represent the approximation exactly.
+    /// assert_eq!("0.100000001490116119384765625", Decimal::from_f32_retain(0.1_f32).unwrap().to_string());
+    /// ```
+    pub fn from_f32_retain(n: f32) -> Option<Self> {
+        from_f32(n, false)
+    }
 
-        while exponent5 > 0 {
-            // Check to see if the mantissa is divisible by 2
-            if bits[0] & 0x1 == 0 {
-                exponent10 += 1;
-                exponent5 -= 1;
-
-                // We can divide by 2 without losing precision
-                let hi_carry = bits[2] & 0x1 == 1;
-                bits[2] >>= 1;
-                let mid_carry = bits[1] & 0x1 == 1;
-                bits[1] = (bits[1] >> 1) | if hi_carry { SIGN_MASK } else { 0 };
-                bits[0] = (bits[0] >> 1) | if mid_carry { SIGN_MASK } else { 0 };
-            } else {
-                // The mantissa is NOT divisible by 2. Therefore the mantissa should
-                // be multiplied by 5, unless the multiplication overflows.
-                exponent5 -= 1;
-
-                let mut temp = [bits[0], bits[1], bits[2]];
-                if ops::array::mul_by_u32(&mut temp, 5) == 0 {
-                    // Multiplication succeeded without overflow, so copy result back
-                    bits[0] = temp[0];
-                    bits[1] = temp[1];
-                    bits[2] = temp[2];
-                } else {
-                    // Multiplication by 5 overflows. The mantissa should be divided
-                    // by 2, and therefore will lose significant digits.
-                    exponent10 += 1;
-
-                    // Shift right
-                    let hi_carry = bits[2] & 0x1 == 1;
-                    bits[2] >>= 1;
-                    let mid_carry = bits[1] & 0x1 == 1;
-                    bits[1] = (bits[1] >> 1) | if hi_carry { SIGN_MASK } else { 0 };
-                    bits[0] = (bits[0] >> 1) | if mid_carry { SIGN_MASK } else { 0 };
-                }
-            }
-        }
-
-        // In order to divide the value by 5, it is best to multiply by 2/10.
-        // Therefore, exponent10 is decremented, and the mantissa should be multiplied by 2
-        while exponent5 < 0 {
-            if bits[2] & SIGN_MASK == 0 {
-                // No far left bit, the mantissa can withstand a shift-left without overflowing
-                exponent10 -= 1;
-                exponent5 += 1;
-                ops::array::shl1_internal(bits, 0);
-            } else {
-                // The mantissa would overflow if shifted. Therefore it should be
-                // directly divided by 5. This will lose significant digits, unless
-                // by chance the mantissa happens to be divisible by 5.
-                exponent5 += 1;
-                ops::array::div_by_u32(bits, 5);
-            }
-        }
-
-        // At this point, the mantissa has assimilated the exponent5, but
-        // exponent10 might not be suitable for assignment. exponent10 must be
-        // in the range [-MAX_PRECISION..0], so the mantissa must be scaled up or
-        // down appropriately.
-        while exponent10 > 0 {
-            // In order to bring exponent10 down to 0, the mantissa should be
-            // multiplied by 10 to compensate. If the exponent10 is too big, this
-            // will cause the mantissa to overflow.
-            if ops::array::mul_by_u32(bits, 10) == 0 {
-                exponent10 -= 1;
-            } else {
-                // Overflowed - return?
-                return None;
-            }
-        }
-
-        // In order to bring exponent up to -MAX_PRECISION, the mantissa should
-        // be divided by 10 to compensate. If the exponent10 is too small, this
-        // will cause the mantissa to underflow and become 0.
-        while exponent10 < -(MAX_PRECISION as i32) {
-            let rem10 = ops::array::div_by_u32(bits, 10);
-            exponent10 += 1;
-            if ops::array::is_all_zero(bits) {
-                // Underflow, unable to keep dividing
-                exponent10 = 0;
-            } else if rem10 >= 5 {
-                ops::array::add_one_internal(bits);
-            }
-        }
-
-        // This step is required in order to remove excess bits of precision from the
-        // end of the bit representation, down to the precision guaranteed by the
-        // floating point number
-        if is64 {
-            // Guaranteed to about 16 dp
-            while exponent10 < 0 && (bits[2] != 0 || (bits[1] & 0xFFF0_0000) != 0) {
-                let rem10 = ops::array::div_by_u32(bits, 10);
-                exponent10 += 1;
-                if rem10 >= 5 {
-                    ops::array::add_one_internal(bits);
-                }
-            }
-        } else {
-            // Guaranteed to about 7 dp
-            while exponent10 < 0 && ((bits[0] & 0xFF00_0000) != 0 || bits[1] != 0 || bits[2] != 0) {
-                let rem10 = ops::array::div_by_u32(bits, 10);
-                exponent10 += 1;
-                if rem10 >= 5 {
-                    ops::array::add_one_internal(bits);
-                }
-            }
-        }
-
-        // Remove multiples of 10 from the representation
-        while exponent10 < 0 {
-            let mut temp = [bits[0], bits[1], bits[2]];
-            let remainder = ops::array::div_by_u32(&mut temp, 10);
-            if remainder == 0 {
-                exponent10 += 1;
-                bits[0] = temp[0];
-                bits[1] = temp[1];
-                bits[2] = temp[2];
-            } else {
-                break;
-            }
-        }
-
-        Some(Decimal {
-            lo: bits[0],
-            mid: bits[1],
-            hi: bits[2],
-            flags: flags(!positive, -exponent10 as u32),
-        })
+    /// Parses a 64-bit float into a Decimal number whilst retaining any non-guaranteed precision.
+    ///
+    /// Typically when a float is parsed in Rust Decimal, any excess bits (after ~15.95 decimal points for
+    /// f64 as per IEEE-754) are removed due to any digits following this are considered an approximation
+    /// at best. This function bypasses this additional step and retains these excess bits.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use rust_decimal::prelude::*;
+    ///
+    /// // Usually floats are parsed leveraging float guarantees. i.e. 0.1_f64 => 0.1
+    /// assert_eq!("0.1", Decimal::from_f64(0.1_f64).unwrap().to_string());
+    ///
+    /// // Sometimes, we may want to represent the approximation exactly.
+    /// assert_eq!("0.1000000000000000055511151231", Decimal::from_f64_retain(0.1_f64).unwrap().to_string());
+    /// ```
+    pub fn from_f64_retain(n: f64) -> Option<Self> {
+        from_f64(n, false)
     }
 
     /// Checked addition. Computes `self + other`, returning `None` if overflow occurred.
@@ -1855,96 +1790,252 @@ impl FromPrimitive for Decimal {
     }
 
     fn from_f32(n: f32) -> Option<Decimal> {
-        // Handle the case if it is NaN, Infinity or -Infinity
-        if !n.is_finite() {
-            return None;
-        }
-
-        // It's a shame we can't use a union for this due to it being broken up by bits
-        // i.e. 1/8/23 (sign, exponent, mantissa)
-        // See https://en.wikipedia.org/wiki/IEEE_754-1985
-        // n = (sign*-1) * 2^exp * mantissa
-        // Decimal of course stores this differently... 10^-exp * significand
-        let raw = n.to_bits();
-        let positive = (raw >> 31) == 0;
-        let biased_exponent = ((raw >> 23) & 0xFF) as i32;
-        let mantissa = raw & 0x007F_FFFF;
-
-        // Handle the special zero case
-        if biased_exponent == 0 && mantissa == 0 {
-            let mut zero = ZERO;
-            if !positive {
-                zero.set_sign_negative(true);
-            }
-            return Some(zero);
-        }
-
-        // Get the bits and exponent2
-        let mut exponent2 = biased_exponent - 127;
-        let mut bits = [mantissa, 0u32, 0u32];
-        if biased_exponent == 0 {
-            // Denormalized number - correct the exponent
-            exponent2 += 1;
-        } else {
-            // Add extra hidden bit to mantissa
-            bits[0] |= 0x0080_0000;
-        }
-
-        // The act of copying a mantissa as integer bits is equivalent to shifting
-        // left the mantissa 23 bits. The exponent is reduced to compensate.
-        exponent2 -= 23;
-
-        // Convert to decimal
-        Decimal::base2_to_decimal(&mut bits, exponent2, positive, false)
+        // By default, we remove excess bits. This allows 0.1_f64 == dec!(0.1).
+        from_f32(n, true)
     }
 
     fn from_f64(n: f64) -> Option<Decimal> {
-        // Handle the case if it is NaN, Infinity or -Infinity
-        if !n.is_finite() {
+        // By default, we remove excess bits. This allows 0.1_f64 == dec!(0.1).
+        from_f64(n, true)
+    }
+}
+
+#[inline]
+fn from_f64(n: f64, remove_excess_bits: bool) -> Option<Decimal> {
+    // Handle the case if it is NaN, Infinity or -Infinity
+    if !n.is_finite() {
+        return None;
+    }
+
+    // It's a shame we can't use a union for this due to it being broken up by bits
+    // i.e. 1/11/52 (sign, exponent, mantissa)
+    // See https://en.wikipedia.org/wiki/IEEE_754-1985
+    // n = (sign*-1) * 2^exp * mantissa
+    // Decimal of course stores this differently... 10^-exp * significand
+    let raw = n.to_bits();
+    let positive = (raw >> 63) == 0;
+    let biased_exponent = ((raw >> 52) & 0x7FF) as i32;
+    let mantissa = raw & 0x000F_FFFF_FFFF_FFFF;
+
+    // Handle the special zero case
+    if biased_exponent == 0 && mantissa == 0 {
+        let mut zero = ZERO;
+        if !positive {
+            zero.set_sign_negative(true);
+        }
+        return Some(zero);
+    }
+
+    // Get the bits and exponent2
+    let mut exponent2 = biased_exponent - 1023;
+    let mut bits = [
+        (mantissa & 0xFFFF_FFFF) as u32,
+        ((mantissa >> 32) & 0xFFFF_FFFF) as u32,
+        0u32,
+    ];
+    if biased_exponent == 0 {
+        // Denormalized number - correct the exponent
+        exponent2 += 1;
+    } else {
+        // Add extra hidden bit to mantissa
+        bits[1] |= 0x0010_0000;
+    }
+
+    // The act of copying a mantissa as integer bits is equivalent to shifting
+    // left the mantissa 52 bits. The exponent is reduced to compensate.
+    exponent2 -= 52;
+
+    // Convert to decimal
+    base2_to_decimal(&mut bits, exponent2, positive, true, remove_excess_bits)
+}
+
+#[inline]
+fn from_f32(n: f32, remove_excess_bits: bool) -> Option<Decimal> {
+    // Handle the case if it is NaN, Infinity or -Infinity
+    if !n.is_finite() {
+        return None;
+    }
+
+    // It's a shame we can't use a union for this due to it being broken up by bits
+    // i.e. 1/8/23 (sign, exponent, mantissa)
+    // See https://en.wikipedia.org/wiki/IEEE_754-1985
+    // n = (sign*-1) * 2^exp * mantissa
+    // Decimal of course stores this differently... 10^-exp * significand
+    let raw = n.to_bits();
+    let positive = (raw >> 31) == 0;
+    let biased_exponent = ((raw >> 23) & 0xFF) as i32;
+    let mantissa = raw & 0x007F_FFFF;
+
+    // Handle the special zero case
+    if biased_exponent == 0 && mantissa == 0 {
+        let mut zero = ZERO;
+        if !positive {
+            zero.set_sign_negative(true);
+        }
+        return Some(zero);
+    }
+
+    // Get the bits and exponent2
+    let mut exponent2 = biased_exponent - 127;
+    let mut bits = [mantissa, 0u32, 0u32];
+    if biased_exponent == 0 {
+        // Denormalized number - correct the exponent
+        exponent2 += 1;
+    } else {
+        // Add extra hidden bit to mantissa
+        bits[0] |= 0x0080_0000;
+    }
+
+    // The act of copying a mantissa as integer bits is equivalent to shifting
+    // left the mantissa 23 bits. The exponent is reduced to compensate.
+    exponent2 -= 23;
+
+    // Convert to decimal
+    base2_to_decimal(&mut bits, exponent2, positive, false, remove_excess_bits)
+}
+
+fn base2_to_decimal(
+    bits: &mut [u32; 3],
+    exponent2: i32,
+    positive: bool,
+    is64: bool,
+    remove_excess_bits: bool,
+) -> Option<Decimal> {
+    // 2^exponent2 = (10^exponent2)/(5^exponent2)
+    //             = (5^-exponent2)*(10^exponent2)
+    let mut exponent5 = -exponent2;
+    let mut exponent10 = exponent2; // Ultimately, we want this for the scale
+
+    while exponent5 > 0 {
+        // Check to see if the mantissa is divisible by 2
+        if bits[0] & 0x1 == 0 {
+            exponent10 += 1;
+            exponent5 -= 1;
+
+            // We can divide by 2 without losing precision
+            let hi_carry = bits[2] & 0x1 == 1;
+            bits[2] >>= 1;
+            let mid_carry = bits[1] & 0x1 == 1;
+            bits[1] = (bits[1] >> 1) | if hi_carry { SIGN_MASK } else { 0 };
+            bits[0] = (bits[0] >> 1) | if mid_carry { SIGN_MASK } else { 0 };
+        } else {
+            // The mantissa is NOT divisible by 2. Therefore the mantissa should
+            // be multiplied by 5, unless the multiplication overflows.
+            exponent5 -= 1;
+
+            let mut temp = [bits[0], bits[1], bits[2]];
+            if ops::array::mul_by_u32(&mut temp, 5) == 0 {
+                // Multiplication succeeded without overflow, so copy result back
+                bits[0] = temp[0];
+                bits[1] = temp[1];
+                bits[2] = temp[2];
+            } else {
+                // Multiplication by 5 overflows. The mantissa should be divided
+                // by 2, and therefore will lose significant digits.
+                exponent10 += 1;
+
+                // Shift right
+                let hi_carry = bits[2] & 0x1 == 1;
+                bits[2] >>= 1;
+                let mid_carry = bits[1] & 0x1 == 1;
+                bits[1] = (bits[1] >> 1) | if hi_carry { SIGN_MASK } else { 0 };
+                bits[0] = (bits[0] >> 1) | if mid_carry { SIGN_MASK } else { 0 };
+            }
+        }
+    }
+
+    // In order to divide the value by 5, it is best to multiply by 2/10.
+    // Therefore, exponent10 is decremented, and the mantissa should be multiplied by 2
+    while exponent5 < 0 {
+        if bits[2] & SIGN_MASK == 0 {
+            // No far left bit, the mantissa can withstand a shift-left without overflowing
+            exponent10 -= 1;
+            exponent5 += 1;
+            ops::array::shl1_internal(bits, 0);
+        } else {
+            // The mantissa would overflow if shifted. Therefore it should be
+            // directly divided by 5. This will lose significant digits, unless
+            // by chance the mantissa happens to be divisible by 5.
+            exponent5 += 1;
+            ops::array::div_by_u32(bits, 5);
+        }
+    }
+
+    // At this point, the mantissa has assimilated the exponent5, but
+    // exponent10 might not be suitable for assignment. exponent10 must be
+    // in the range [-MAX_PRECISION..0], so the mantissa must be scaled up or
+    // down appropriately.
+    while exponent10 > 0 {
+        // In order to bring exponent10 down to 0, the mantissa should be
+        // multiplied by 10 to compensate. If the exponent10 is too big, this
+        // will cause the mantissa to overflow.
+        if ops::array::mul_by_u32(bits, 10) == 0 {
+            exponent10 -= 1;
+        } else {
+            // Overflowed - return?
             return None;
         }
-
-        // It's a shame we can't use a union for this due to it being broken up by bits
-        // i.e. 1/11/52 (sign, exponent, mantissa)
-        // See https://en.wikipedia.org/wiki/IEEE_754-1985
-        // n = (sign*-1) * 2^exp * mantissa
-        // Decimal of course stores this differently... 10^-exp * significand
-        let raw = n.to_bits();
-        let positive = (raw >> 63) == 0;
-        let biased_exponent = ((raw >> 52) & 0x7FF) as i32;
-        let mantissa = raw & 0x000F_FFFF_FFFF_FFFF;
-
-        // Handle the special zero case
-        if biased_exponent == 0 && mantissa == 0 {
-            let mut zero = ZERO;
-            if !positive {
-                zero.set_sign_negative(true);
-            }
-            return Some(zero);
-        }
-
-        // Get the bits and exponent2
-        let mut exponent2 = biased_exponent - 1023;
-        let mut bits = [
-            (mantissa & 0xFFFF_FFFF) as u32,
-            ((mantissa >> 32) & 0xFFFF_FFFF) as u32,
-            0u32,
-        ];
-        if biased_exponent == 0 {
-            // Denormalized number - correct the exponent
-            exponent2 += 1;
-        } else {
-            // Add extra hidden bit to mantissa
-            bits[1] |= 0x0010_0000;
-        }
-
-        // The act of copying a mantissa as integer bits is equivalent to shifting
-        // left the mantissa 52 bits. The exponent is reduced to compensate.
-        exponent2 -= 52;
-
-        // Convert to decimal
-        Decimal::base2_to_decimal(&mut bits, exponent2, positive, true)
     }
+
+    // In order to bring exponent up to -MAX_PRECISION, the mantissa should
+    // be divided by 10 to compensate. If the exponent10 is too small, this
+    // will cause the mantissa to underflow and become 0.
+    while exponent10 < -(MAX_PRECISION_U32 as i32) {
+        let rem10 = ops::array::div_by_u32(bits, 10);
+        exponent10 += 1;
+        if ops::array::is_all_zero(bits) {
+            // Underflow, unable to keep dividing
+            exponent10 = 0;
+        } else if rem10 >= 5 {
+            ops::array::add_one_internal(bits);
+        }
+    }
+
+    if remove_excess_bits {
+        // This step is required in order to remove excess bits of precision from the
+        // end of the bit representation, down to the precision guaranteed by the
+        // floating point number (see IEEE-754).
+        if is64 {
+            // Guaranteed to approx 15/16 dp
+            while exponent10 < 0 && (bits[2] != 0 || (bits[1] & 0xFFF0_0000) != 0) {
+                let rem10 = ops::array::div_by_u32(bits, 10);
+                exponent10 += 1;
+                if rem10 >= 5 {
+                    ops::array::add_one_internal(bits);
+                }
+            }
+        } else {
+            // Guaranteed to about 7/8 dp
+            while exponent10 < 0 && ((bits[0] & 0xFF00_0000) != 0 || bits[1] != 0 || bits[2] != 0) {
+                let rem10 = ops::array::div_by_u32(bits, 10);
+                exponent10 += 1;
+                if rem10 >= 5 {
+                    ops::array::add_one_internal(bits);
+                }
+            }
+        }
+
+        // Remove multiples of 10 from the representation
+        while exponent10 < 0 {
+            let mut temp = [bits[0], bits[1], bits[2]];
+            let remainder = ops::array::div_by_u32(&mut temp, 10);
+            if remainder == 0 {
+                exponent10 += 1;
+                bits[0] = temp[0];
+                bits[1] = temp[1];
+                bits[2] = temp[2];
+            } else {
+                break;
+            }
+        }
+    }
+
+    Some(Decimal {
+        lo: bits[0],
+        mid: bits[1],
+        hi: bits[2],
+        flags: flags(!positive, -exponent10 as u32),
+    })
 }
 
 impl ToPrimitive for Decimal {
@@ -1999,7 +2090,10 @@ impl ToPrimitive for Decimal {
 
     fn to_f64(&self) -> Option<f64> {
         if self.scale() == 0 {
-            let integer = self.to_i64();
+            // If scale is zero, we are storing a 96-bit integer value, that would
+            // always fit into i128, which in turn is always representable as f64,
+            // albeit with loss of precision for values outside of -2^53..2^53 range.
+            let integer = self.to_i128();
             integer.map(|i| i as f64)
         } else {
             let sign: f64 = if self.is_sign_negative() { -1.0 } else { 1.0 };
@@ -2053,8 +2147,13 @@ impl core::convert::TryFrom<Decimal> for f64 {
 
 impl fmt::Display for Decimal {
     fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), fmt::Error> {
-        let rep = crate::str::to_str_internal(self, false, f.precision());
-        f.pad_integral(self.is_sign_positive(), "", rep.as_str())
+        let (rep, additional) = crate::str::to_str_internal(self, false, f.precision());
+        if let Some(additional) = additional {
+            let value = [rep.as_str(), "0".repeat(additional).as_str()].concat();
+            f.pad_integral(self.is_sign_positive(), "", value.as_str())
+        } else {
+            f.pad_integral(self.is_sign_positive(), "", rep.as_str())
+        }
     }
 }
 
