@@ -15,8 +15,8 @@
 //!
 //! ```toml
 //! [dependencies]
-//! rust_decimal = "1.16"
-//! rust_decimal_macros = "1.16"
+//! rust_decimal = "1.19"
+//! rust_decimal_macros = "1.19"
 //! ```
 //!
 //! ## Usage
@@ -79,6 +79,7 @@
 //! * [db-diesel-postgres](#db-diesel-postgres)
 //! * [legacy-ops](#legacy-ops)
 //! * [maths](#maths)
+//! * [rocket-traits](#rocket-traits)
 //! * [rust-fuzz](#rust-fuzz)
 //! * [serde-float](#serde-float)
 //! * [serde-str](#serde-str)
@@ -111,12 +112,16 @@
 //!
 //! The `maths` feature enables additional complex mathematical functions such as `pow`, `ln`, `enf`, `exp` etc.
 //! Documentation detailing the additional functions can be found on the
-//! [`MathematicalOps`](https://docs.rs/rust_decimal/latest/rust_decimal/trait.MathematicalOps.html) trait.  
+//! [`MathematicalOps`](https://docs.rs/rust_decimal/latest/rust_decimal/trait.MathematicalOps.html) trait.
 //!
 //! Please note that `ln` and `log10` will panic on invalid input with `checked_ln` and `checked_log10` the preferred functions
 //! to curb against this. When the `maths` feature was first developed the library would return `0` on invalid input. To re-enable this
 //! non-panicing behavior, please use the feature: `maths-nopanic`.
 //!
+//! ### `rocket-traits`
+//!
+//! Enable support for forms by implementing the `FromFormField` trait.
+//!
 //! ### `rust-fuzz`
 //!
 //! Enable `rust-fuzz` support by implementing the `Arbitrary` trait.
@@ -170,12 +175,20 @@ mod error;
 mod ops;
 mod str;
 
-#[cfg(any(feature = "postgres", feature = "diesel"))]
-mod db;
 #[cfg(feature = "rust-fuzz")]
 mod fuzz;
 #[cfg(feature = "maths")]
 mod maths;
+#[cfg(any(feature = "db-diesel-mysql"))]
+mod mysql;
+#[cfg(any(
+    feature = "db-tokio-postgres",
+    feature = "db-postgres",
+    feature = "db-diesel-postgres",
+))]
+mod postgres;
+#[cfg(feature = "rocket-traits")]
+mod rocket;
 #[cfg(feature = "serde")]
 mod serde;
 
@@ -190,7 +203,7 @@ pub mod prelude {
     pub use crate::maths::MathematicalOps;
     pub use crate::{Decimal, RoundingStrategy};
     pub use core::str::FromStr;
-    pub use num_traits::{FromPrimitive, One, ToPrimitive, Zero};
+    pub use num_traits::{FromPrimitive, One, Signed, ToPrimitive, Zero};
 }
 
 #[cfg(feature = "diesel")]
